/*
-------------------------------------------------------------------------------
    This file is a part of the Vst Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: paramwidget.cpp
  Author: Francois Mazen
  Date: 28/05/07 16:25
  Description: a paramwidget is a widget which contains bouton, label and 
               functions, in order to get the random parameters.
-------------------------------------------------------------------------------
*/

#include <QtGui>
#include "paramwidget.h"


//-----------------------------------------------------------------------------
ParamWidget::ParamWidget(qint32 num, VpgRandom *random_module, QString param_name): QWidget()
{
       randomModule = random_module;
       container = new QHBoxLayout(this);
       
       name = param_name;
       
       //labels
       QString charbuffer;
       charbuffer.setNum(num);
       charbuffer.append(" ");
       charbuffer.append(name);
       charbuffer.prepend( tr("#"));
       lblParam = new QLabel(charbuffer);
       lbl1 = new QLabel(tr("min:"),this);
       lbl1->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
       lbl2 = new QLabel(tr("max:"),this);
       lbl2->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
       
       //combo box
       cbRandomType = new QComboBox(this);
       cbRandomType->addItem(tr("Uniform Random"), UNIFORM_RANDOM); //0
       cbRandomType->addItem(tr("Normal Random"), NORMAL_RANDOM);  //1
       cbRandomType->addItem(tr("Constant"), CONSTANT);       //2
       

       
       //spin boxes
       spbx1 = new QDoubleSpinBox(this);
       spbx1->setRange(0,1);
       spbx1->setSingleStep(0.1); 
       spbx2 = new QDoubleSpinBox(this);
       spbx2->setRange(0,1);
       spbx2->setSingleStep(0.1); 
       spbx2->setValue(1);
       
       //fill container
       container->addWidget(lblParam);
       container->addWidget(cbRandomType);
       container->addWidget(lbl1);
       container->addWidget(spbx1);
       container->addWidget(lbl2);
       container->addWidget(spbx2);
       container->setMargin(0);
       
       setLayout(container);
       
       //Qt connections
       connect(cbRandomType, SIGNAL(activated(int)), this, SLOT(SetState(int)));
       connect(spbx1, SIGNAL(valueChanged(double)), this, SLOT(checkSpinBox()));
       connect(spbx2, SIGNAL(valueChanged(double)), this, SLOT(checkSpinBox()));
       
      
       
       
}



//-----------------------------------------------------------------------------
void ParamWidget::checkSpinBox()
{
      int type = cbRandomType->currentIndex();
      if (type != UNIFORM_RANDOM)
         return;
        
      if(spbx1->value() > spbx2->value())
      {
             spbx2->setValue(spbx1->value());
      }
      if(spbx2->value() < spbx1->value())
      {
             spbx1->setValue(spbx2->value());
      }
}

//-----------------------------------------------------------------------------
void ParamWidget::SetState(int t_state)
{
     
     switch(t_state)
     {
           case UNIFORM_RANDOM: 
                lbl1->setText(tr("min:"));
                lbl2->setText(tr("max:"));
                lbl2->setEnabled(true);
                
                spbx1->setRange(0,1);
                spbx1->setSingleStep(0.1); 
                spbx1->setValue(0);
                spbx2->setRange(0,1);
                spbx2->setSingleStep(0.1); 
                spbx2->setValue(1);
                spbx2->setEnabled(true);
                break;
                
           case CONSTANT: 
                lbl1->setText(tr("value:"));
                spbx1->setRange(0,1);
                spbx1->setSingleStep(0.1); 
                spbx1->setValue(0);
                //disable second spinbox:
                lbl2->setText("");
                lbl2->setEnabled(false);
                spbx2->setEnabled(false);
                break;
                
                
           case NORMAL_RANDOM:
                lbl1->setText(tr("mid:"));
                spbx1->setRange(0,1);
                spbx1->setSingleStep(0.1); 
                spbx1->setValue(0.5);
                
                lbl2->setText(tr("dev:"));
                lbl2->setEnabled(true);
                spbx2->setRange(0,1);
                spbx2->setSingleStep(0.01); 
                spbx2->setValue(0.1);
                spbx2->setEnabled(true);
                break;
           default:
                   break;

     }    
     
}

//-----------------------------------------------------------------------------
float ParamWidget::getRandomValue()
{
      int type = cbRandomType->currentIndex();
      
      switch(type)
      {
      case CONSTANT:
           return (spbx1->value());
           break;
      case NORMAL_RANDOM:
           //mu and deviation
           return (randomModule->getNormalValue(spbx1->value(), spbx2->value()));
           break;
      case UNIFORM_RANDOM:
           //min-max
      default:
           return (randomModule->getUniformValue(spbx1->value(), spbx2->value()));
           break;
      
      }
      
      return 0.f;
}
